//------------------------- Verticle SCROLLER ---------------------------
//                by Carl Paulson      Cognitive Portals
//                 http://members.tripod.com/~SleepyCP/
//                          cpaulson@home.com
//-----------------------------------------------------------------------
//
//  Suggested HTML Input:
/*
        <APPLET code="Scroller.class" width=300 height=200>
            <param name=NoOfLines value="8">
            <param name=MSG0 value="I'm a rambler. I'm a gambler.">
            <param name=MSG1 value="I'm a long way from home.">
            <param name=MSG2 value="And if you don't like me,">
            <param name=MSG3 value="well, leave me alone.">
            <param name=MSG4 value="I'll eat when I'm hungry.">
            <param name=MSG5 value="I'll drink when I'm dry.">
            <param name=MSG6 value="And if moonshine don't kill me,">
            <param name=MSG7 value="I'll live till I die.">

            <param name=FONT       value="Helvetica">
            <param name=STYLE      value="1">
            <param name=SIZE       value="18">

            <param name=StepIn     value="1">
            <param name=PauseIn    value="70">

            <param name=BG_Color   value="blue">
            <param name=FG_Color   value="red">

            <CENTER><FONT SIZE=3>{If you can read this, your browser 
            dosen`t support Java.}</FONT><BR>
            <FONT FACE="Helvetica" COLOR="#FF0000" SIZE=7>Too Bad!!  
            You're missing a good drinking song!</FONT></CENTER><BR>
        </APPLET>
*/
//
//-----------------------------------------------------------------------

import java.applet.Applet;
import java.util.*;
import java.awt.*;


public class Scroller extends Applet implements Runnable {

    private String   MsgName,   msg[];
    private int      vertStep,  x[];
    private int      pause,     y[];
    private int      Height1,   Width1, fontHeight;
    public  Thread   myThread;
    private int      msgWidth[];
    private int      msgNumber, iCount, NumberShow, NumberTot;
    private boolean  Bool1;
    private Graphics gBuffer;
    private Image    iBuffer;
    private FontMetrics fm;
    private Color    ColorBG,   ColorFG;

    public void init() {

        setFont(getFontParam("FONT","Helvetica","STYLE",1,"SIZE",24));

        Width1     = size().width;
        Height1    = size().height;

        fm = getFontMetrics(getFont());
        fontHeight = fm.getHeight();

        msgNumber  = getIntParam("NoOfLines",1,1,12);
        msg        = new String[msgNumber];
        msgWidth   = new int[msgNumber];
        x          = new int[msgNumber];
        y          = new int[msgNumber];
        iCount     = 0;

        for (int i = 0; i < msgNumber; i++) {
            String MsgName = "MSG" + i;
            msg[i]         = getStringParam(MsgName," ");
            msgWidth[i]    = fm.stringWidth(msg[i]);
            x[i]           = (Width1 - msgWidth[i])/2;
            y[i]           = i*(fontHeight + 5) + 5;
        }

        NumberShow = size().height/(fontHeight + 5);
        vertStep   = getIntParam("StepIn",1,1,2);
        NumberTot  = (msgNumber + NumberShow)*(fontHeight + 5)/vertStep; // pixels
        pause      = getIntParam("PauseIn",100,0,5000);

        ColorFG    = getColorParam("FG_Color", "BLUE");
        setForeground(ColorFG);
        ColorBG    = getColorParam("BG_Color", "WHITE");
        setBackground(ColorBG);

    }

    // ------------------------------------------------------------------

    public void start() {
        if (!Bool1) {
            if (myThread == null) myThread = new Thread(this);
            myThread.start();
        }
    }

    //-------------------------------------------------------------------

    public void stop() {
        myThread = null;
        repaint();
    }

    //-------------------------------------------------------------------
                            /** @deprecated */
    public boolean mouseDown(Event event, int i, int j) {
        if (Bool1) {
            Bool1 = false;
            start();
        } else {
            Bool1 = true;
            stop();
        }
        return true;
    }

    // --------------------- implement Runnable -------------------------

    public void run() {	

        while (myThread != null) {
            try {
                myThread.sleep (pause);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            repaint();

        }
    }

    // ---------------------  Create a buffer --=------------------------

    public void createBuffers() { 
        iBuffer = createImage(Width1, Height1);
        gBuffer = iBuffer.getGraphics();
    }

    // -------------- Override: Prevent Clearing Screen -----------------

    public void update(Graphics g) { 
        paint(g);
    }

    // ------------------------------------------------------------------

    public synchronized void paint(Graphics g) {

        if(gBuffer == null) createBuffers();
        gBuffer.setColor(ColorBG);
        gBuffer.fillRect(0, 0, Width1, Height1);

        int    x1;
        int    y1;

        for (int j=0; j < msgNumber; j++) {
            x1 = x[j];
            y1 = y[j] + size().height - iCount*vertStep;
//          System.out.println("iCount,j = "+iCount+"  "+j); //CCP
            gBuffer.setColor(ColorFG);
            gBuffer.drawString(msg[j], x1, y1);
        }
        g.drawImage(iBuffer, 0, 0, this);
        if (iCount < NumberTot) iCount++;
        else iCount = 0;
    }

    // ------------------------ getString -------------------------------

    public String getStringParam(String x, String def_X) {
        String str;
        str = getParameter(x);
        if (str == null) str = def_X;
        return str;
    }

    // ----------------------- getInteger -------------------------------

    public int getIntParam(String i, int def_I, int min_i, int max_i) {
        int i_in;
        try {
            i_in = Integer.parseInt(getParameter(i));
            if ((i_in < min_i)||(i_in > max_i)) throw new Exception();
        } catch (Exception e) {
            System.out.println(i+" invalid; using default value: "+def_I);
            i_in = def_I;
        }	
        return i_in;
    }

    // ------------------------ getColor --------------------------------

    public Color getColorParam(String c, String def_C) {

        Color Color1 = Color.lightGray;      // default color
        if (def_C != null) {
            if      (def_C.equalsIgnoreCase("black"))     Color1 = Color.black;
            else if (def_C.equalsIgnoreCase("blue"))      Color1 = Color.blue;
            else if (def_C.equalsIgnoreCase("cyan"))      Color1 = Color.cyan;
            else if (def_C.equalsIgnoreCase("darkGray"))  Color1 = Color.darkGray;
            else if (def_C.equalsIgnoreCase("gray"))      Color1 = Color.gray;
            else if (def_C.equalsIgnoreCase("green"))     Color1 = Color.green;
            else if (def_C.equalsIgnoreCase("lightGray")) Color1 = Color.lightGray;
            else if (def_C.equalsIgnoreCase("magenta"))   Color1 = Color.magenta;
            else if (def_C.equalsIgnoreCase("orange"))    Color1 = Color.orange;
            else if (def_C.equalsIgnoreCase("pink"))      Color1 = Color.pink;
            else if (def_C.equalsIgnoreCase("red"))       Color1 = Color.red;
            else if (def_C.equalsIgnoreCase("white"))     Color1 = Color.white;
            else if (def_C.equalsIgnoreCase("yellow"))    Color1 = Color.yellow;
        }

        StringTokenizer st;
        String rgbDelim = ":,.";
        String c_str = getParameter(c);
        if (c_str != null) st = new StringTokenizer(c_str, rgbDelim);
        else return Color1;

        if      (c_str.equalsIgnoreCase("black"))     return Color.black;
        else if (c_str.equalsIgnoreCase("blue"))      return Color.blue;
        else if (c_str.equalsIgnoreCase("cyan"))      return Color.cyan;
        else if (c_str.equalsIgnoreCase("darkGray"))  return Color.darkGray;
        else if (c_str.equalsIgnoreCase("gray"))      return Color.gray;
        else if (c_str.equalsIgnoreCase("green"))     return Color.green;
        else if (c_str.equalsIgnoreCase("lightGray")) return Color.lightGray;
        else if (c_str.equalsIgnoreCase("magenta"))   return Color.magenta;
        else if (c_str.equalsIgnoreCase("orange"))    return Color.orange;
        else if (c_str.equalsIgnoreCase("pink"))      return Color.pink;
        else if (c_str.equalsIgnoreCase("red"))       return Color.red;
        else if (c_str.equalsIgnoreCase("white"))     return Color.white;
        else if (c_str.equalsIgnoreCase("yellow"))    return Color.yellow;

        else if (st.countTokens() == 3) {
            Integer r = new Integer(st.nextToken());
            Integer g = new Integer(st.nextToken());
            Integer b = new Integer(st.nextToken());
            Color1    = new Color(r.intValue(), g.intValue(), b.intValue());
            return Color1;
            }
        return Color1;
    }

    // -------------------------- getFont  ------------------------------

    public Font getFontParam(String fnt, String def_Fnt, 
            String Style, int def_Style, String Size, int def_Size) {

        String font_Name  = getStringParam(fnt,   def_Fnt);
        int    font_Style = getIntParam   (Style, def_Style, 0, 3);
        int    font_Size  = getIntParam   (Size,  def_Size, 4, 100);

        return (new Font(font_Name, font_Style, font_Size));
    }

    // ------------------------------------------------------------------

}

